chiark / gitweb /
udevd: improve timeout handling
[elogind.git] / udev.c
diff --git a/udev.c b/udev.c
index 069cecf973193212f82a89a7c0743e6d256a83fc..2be4c357408d7f266da37a3b4303071f7822bfff 100644 (file)
--- a/udev.c
+++ b/udev.c
 #include "udev_rules.h"
 #include "logging.h"
 
-
 #ifdef USE_LOG
-void log_message(int level, const char *format, ...)
+void log_message(int priority, const char *format, ...)
 {
        va_list args;
 
-       if (!udev_log)
+       if (priority > udev_log_priority)
                return;
 
        va_start(args, format);
-       vsyslog(level, format, args);
+       vsyslog(priority, format, args);
        va_end(args);
 }
 #endif
 
/* decide if we should manage the whole hotplug event
* for now look if the kernel calls udevsend instead of /sbin/hotplug
  */
 static int manage_hotplug_event(void) {
        char helper[256];
@@ -71,13 +72,15 @@ static int manage_hotplug_event(void) {
        if (fd < 0)
                return 0;
 
-       len = read(fd, helper, 256);
+       len = read(fd, helper, sizeof(helper)-1);
        close(fd);
 
        if (len < 0)
                return 0;
        helper[len] = '\0';
 
+       if (helper[0] == '\0' || helper[0] == '\n')
+               return 1;
        if (strstr(helper, "udevsend"))
                return 1;
 
@@ -115,13 +118,12 @@ int main(int argc, char *argv[], char *envp[])
        }
 
        logging_init("udev");
-       dbg("version %s", UDEV_VERSION);
-
        udev_init_config();
+       dbg("version %s", UDEV_VERSION);
 
        /* set signal handlers */
        memset(&act, 0x00, sizeof(act));
-       act.sa_handler = (void (*) (int))sig_handler;
+       act.sa_handler = (void (*)(int)) sig_handler;
        sigemptyset (&act.sa_mask);
        act.sa_flags = 0;
        sigaction(SIGALRM, &act, NULL);
@@ -143,18 +145,24 @@ int main(int argc, char *argv[], char *envp[])
        if (!subsystem && argc == 2)
                subsystem = argv[1];
 
-       udev_init_device(&udev, devpath, subsystem);
+       udev_init_device(&udev, devpath, subsystem, action);
 
        if (!action || !subsystem || !devpath) {
-               dbg("action, subsystem or devpath missing");
+               err("action, subsystem or devpath missing");
                goto hotplug;
        }
 
        /* export logging flag, as called scripts may want to do the same as udev */
-       if (udev_log)
-               setenv("UDEV_LOG", "1", 1);
+       if (udev_log_priority) {
+               char priority[32];
+
+               sprintf(priority, "%i", udev_log_priority);
+               setenv("UDEV_LOG", priority, 1);
+       }
 
        if (udev.type == DEV_BLOCK || udev.type == DEV_CLASS || udev.type == DEV_NET) {
+               udev_rules_init();
+
                if (strcmp(action, "add") == 0) {
                        /* wait for sysfs and possibly add node */
                        dbg("udev add");
@@ -169,16 +177,13 @@ int main(int argc, char *argv[], char *envp[])
                        path[sizeof(path)-1] = '\0';
                        class_dev = wait_class_device_open(path);
                        if (class_dev == NULL) {
-                               dbg ("open class device failed");
+                               dbg("open class device failed");
                                goto hotplug;
                        }
                        dbg("opened class_dev->name='%s'", class_dev->name);
 
                        wait_for_class_device(class_dev, &error);
 
-                       /* init rules */
-                       udev_rules_init();
-
                        /* name, create node, store in db */
                        retval = udev_add_device(&udev, class_dev);
 
@@ -193,16 +198,31 @@ int main(int argc, char *argv[], char *envp[])
                                goto hotplug;
                        }
 
+                       udev_rules_get_run(&udev);
+                       if (udev.ignore_device) {
+                               dbg("device event will be ignored");
+                               goto hotplug;
+                       }
+
                        /* get node from db, remove db-entry, delete created node */
                        retval = udev_remove_device(&udev);
                }
 
-               /* run dev.d/ scripts if we created/deleted a node or changed a netif name */
-               if (udev.devname[0] != '\0') {
+               if (udev.devname[0] != '\0')
                        setenv("DEVNAME", udev.devname, 1);
-                       if (udev_dev_d)
-                               udev_multiplex_directory(&udev, DEVD_DIR, DEVD_SUFFIX);
+
+               if (udev_run && !list_empty(&udev.run_list)) {
+                       struct name_entry *name_loop;
+
+                       dbg("executing run list");
+                       list_for_each_entry(name_loop, &udev.run_list, node)
+                               execute_command(name_loop->name, udev.subsystem);
                }
+
+               /* run dev.d/ scripts if we created/deleted a node or changed a netif name */
+               if (udev_dev_d && udev.devname[0] != '\0')
+                       udev_multiplex_directory(&udev, DEVD_DIR, DEVD_SUFFIX);
+
        } else if (udev.type == DEV_DEVICE) {
                if (strcmp(action, "add") == 0) {
                        /* wait for sysfs */
@@ -228,10 +248,11 @@ int main(int argc, char *argv[], char *envp[])
        }
 
 hotplug:
-       udev_cleanup_device(&udev);
        if (udev_hotplug_d && managed_event)
                udev_multiplex_directory(&udev, HOTPLUGD_DIR, HOTPLUG_SUFFIX);
 
+       udev_cleanup_device(&udev);
+
        logging_close();
        return retval;
 }
@@
 #endif
 
-/* decide if we should manage the whole hotplug event
- * for now look if the kernel calls udevsend instead of /sbin/hotplug
+/* Decide if we should manage the whole uevent, including multiplexing
+ * of the hotplug directories.
+ * For now look if the kernel calls udevsend instead of /sbin/hotplug,
+ * or the uevent-helper in /proc/sys/kernel/hotplug is empty.
  */
 static int manage_hotplug_event(void) {